///////////////////////////////////////////////////////////////////////
// CW Decoder made by Hjalmar Skovholm Hansen OZ1JHM  VER 1.01       //
// Feel free to change, copy or what ever you like but respect       //
// that license is http://www.gnu.org/copyleft/gpl.html              //
// Discuss and give great ideas on                                   //
// https://groups.yahoo.com/neo/groups/oz1jhm/conversations/messages //
//                                                                   //
// Modifications for Japanese Morse code "decoder_kh.ino"            //
//                    JH1LHV Mr.CHIBA                                //
//-------------------------------------------------------------------//
// Modifications by JG1CCL W3CCL Ver.1.01-1R5 Oct 1,2021             //
//  Leave history on serial monitor                                  //
// Modifications by JG1CCL W3CCL Ver.1.01-1R4 May 13,2021            //
//  Save the setting data to EEPROM                                  //
// Modifications by JG1CCL W3CCL Ver.1.01-1R3 May 12,2021            //
//  Chattering prevention and status display                         //
// Modifications by JG1CCL W3CCL Ver.1.01-1R2 March 16,2021          //
//  letter space, word space                                         //
//  For letter or word delimiters, display the letter first          // 
// Modifications by JG1CCL W3CCL Ver.1.01-1R1 March 14,2021          //
//  「→」：欧文→和文"-..---"、「←」：欧文←和文"...-."            //
// Modifications by JG1CCL W3CCL Ver.1.01-1R0 February 25,2021       //
//  Changed to work on Arduino Nano Every                            //
//                    and on LCD with I2C interface                  //
//  Added selection of "Target Frequency" and "Bandwidth"            //
//  Added target_freqArray[],nArray[],BasicGoertzelCal()             //
///////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////
// Read more here http://en.wikipedia.org/wiki/Goertzel_algorithm        //
// if you want to know about FFT the http://www.dspguide.com/pdfbook.htm //
///////////////////////////////////////////////////////////////////////////

#include <LiquidCrystal.h>
//#include <LiquidCrystal_I2C.h>  //-----1R0 I2C LCD by JG1CCL W3CCL
#include <EEPROM.h>             //-----1R4 EEPROM by JG1CCL W3CCL

//-----1R0 SW by JG1CCL W3CCL
#define FLAG_ON 1
#define FLAG_OFF 0
//-----1R0 SW by JG1CCL W3CCL
//-----1R3 SW by JG1CCL W3CCL
#define PUSH_SHORT 5
#define PUSH_LONG 30
//-----1R3 SW by JG1CCL W3CCL
//-----1R4 EEPROM by JG1CCL W3CCL
#define EEPROM_ADDRESS_SW1 0  // EEPROM Address SW1
#define EEPROM_ADDRESS_SW2 1  // EEPROM Address SW2
#define EEPROM_ADDRESS_SW3 2  // EEPROM Address SW3
#define DEFAULT_SW1 0         // 欧文
#define DEFAULT_SW2 1         // Target Frequancy = 558.0 adjust for your needs see above
#define DEFAULT_SW3 1         // n = 48.0 if you change her please change next line also
                              // However, it will be set to the data saved in EEPROM 
//-----1R4 EEPROM by JG1CCL W3CCL

///////////////////////////////////////////////
// select the pins used on the LCD panel      /
///////////////////////////////////////////////
//  LiquidCrystal lcd(RS, E, D4, D5, D6, D7) //
///////////////////////////////////////////////

LiquidCrystal lcd(8, 9, 4, 5, 6, 7);
const int colums = 16;                      // have to be 16 or 20
const int rows = 2;                         // have to be 2 or 4
//LiquidCrystal_I2C lcd(0x27, colums, rows);  //-----1R0 I2C LCD by JG1CCL W3CCL

int lcdindex = 0;
int line1[colums];
int line2[colums];
int buttonFlag1 = FLAG_OFF; //-----1R0 SW by JG1CCL W3CCL
int buttonFlag2 = FLAG_OFF; //-----1R0 SW by JG1CCL W3CCL
int buttonFlag3 = FLAG_OFF; //-----1R0 SW by JG1CCL W3CCL
long capacity = 0;          //-----1R3 Capacity counter by JG1CCL W3CCL
  
////////////////////////////////
// Define 8 specials letters  //
////////////////////////////////
byte U_umlaut[8]   = {B01010,B00000,B10001,B10001,B10001,B10001,B01110,B00000}; // 'Ü'
byte O_umlaut[8]   = {B01010,B00000,B01110,B10001,B10001,B10001,B01110,B00000}; // 'Ö'
byte A_umlaut[8]   = {B01010,B00000,B01110,B10001,B11111,B10001,B10001,B00000}; // 'Ä'
byte AE_capital[8] = {B01111,B10100,B10100,B11110,B10100,B10100,B10111,B00000}; // 'Æ'
byte OE_capital[8] = {B00001,B01110,B10011,B10101,B11001,B01110,B10000,B00000}; // 'Ø'
byte fullblock[8]  = {B11111,B11111,B11111,B11111,B11111,B11111,B11111,B11111};
byte AA_capital[8] = {B00100,B00000,B01110,B10001,B11111,B10001,B10001,B00000}; // 'Å'
byte emtyblock[8]  = {B00000,B00000,B00000,B00000,B00000,B00000,B00000,B00000};
int audioInPin = A1;
int audioOutPin = 11;
int ledPin = 13;
int sw1Pin = 10;  //-----1R0 SW スイッチ1の接続ピンをD10に設定 by JG1CCL W3CCL
int sw2Pin = 3;   //-----1R0 SW スイッチ2の接続ピンをD3に設定 by JG1CCL W3CCL
int sw3Pin = 2;   //-----1R0 SW スイッチ3の接続ピンをD2に設定 by JG1CCL W3CCL

float magnitude;
int magnitudelimit = 100;
int magnitudelimit_low = 100;
int realstate = LOW;
int realstatebefore = LOW;
int filteredstate = LOW;
int filteredstatebefore = LOW;

///////////////////////////////////////////////////////////
// The sampling frq will be 8928 on a 16 mhz             //
// without any prescaler etc                             //
// because we need the tone in the center of the bins    //
// you can set the tone to 496, 558, 744 or 992          //
// then n the number of samples which give the bandwidth //
// can be (8928 / tone) * 1 or 2 or 3 or 4 etc           //
// init is 8928/558 = 16 *4 = 64 samples                 //
// try to take n = 96 or 128 ;o)                         //
// 48 will give you a bandwidth around 186 hz            //
// 64 will give you a bandwidth around 140 hz            //
// 96 will give you a bandwidth around 94 hz             //
// 128 will give you a bandwidth around 70 hz            //
// BUT remember that high n take a lot of time           //
// so you have to find the compromice - i use 48         //
///////////////////////////////////////////////////////////
float coeff;
float Q1 = 0;
float Q2 = 0;
float sine;
float cosine;  
float sampling_freq = 8928.0;

//-----1R0 SW by JG1CCL W3CCL
//float target_freq = 558.0;  // adjust for your needs see above
//float n = 48.0;             // if you change her please change next line also
float target_freq;
float n;
//-----1R0 SW by JG1CCL W3CCL

//-----1R0 SW 配列に変更 by JG1CCL W3CCL
//int testData[48];
float target_freqArray[] = {496.0, 558.0, 744.0, 992.0};  // Target Frequency Array
float nArray[] = {32.0, 48.0, 64.0, 96.0};                // n Array
int testData[96];
float bw;                                                 // Bandwidth
//-----1R0 SW by JG1CCL W3CCL

//////////////////////////////
// Noise Blanker time which //
// shall be computed so     //
// this is initial          //
//////////////////////////////
int nbtime = 6; // ms noise blanker         

long starttimehigh;
long highduration;
long lasthighduration;
long hightimesavg;
long lowtimesavg;
long startttimelow;
long lowduration;
long laststarttime = 0;

char code[20];
int stop = LOW;
int wpm;

//-----1R4 EEPROM by JG1CCL W3CCL
//int sw = 0;   // この変数宣言と初期値を追加してください
//int sw2 = 1;  //-----1R0 SW Target Frequancy = 558.0 by JG1CCL W3CCL
//int sw3 = 1;  //-----1R0 SW n = 48.0 by JG1CCL W3CCL
int sw;
int sw2;
int sw3;
//-----1R4 EEPROM by JG1CCL W3CCL

////////////////
// init setup //
////////////////
void setup() {
  pinMode(sw1Pin, INPUT_PULLUP);  //-----1R0 SW スイッチ1の接続ピンをデジタル入力に設定 by JG1CCL W3CCL
  pinMode(sw2Pin, INPUT_PULLUP);  //-----1R0 SW スイッチ2の接続ピンをデジタル入力に設定 by JG1CCL W3CCL
  pinMode(sw3Pin, INPUT_PULLUP);  //-----1R0 SW スイッチ3の接続ピンをデジタル入力に設定 by JG1CCL W3CCL
  pinMode(ledPin, OUTPUT);        //-----1R0 SW LEDに接続ピンをデジタル出力に設定 by JG1CCL W3CCL

//-----1R4 EEPROM by JG1CCL W3CCL
  sw = EEPROM.read(EEPROM_ADDRESS_SW1);
  if(sw > 1)sw = DEFAULT_SW1;
  sw2 = EEPROM.read(EEPROM_ADDRESS_SW2);
  if(sw2 > 3)sw2 = DEFAULT_SW2;
  target_freq = target_freqArray[sw2];
  sw3 = EEPROM.read(EEPROM_ADDRESS_SW3);
  if(sw3 > 3)sw3 = DEFAULT_SW3;
  n = nArray[sw3];
//-----1R4 EEPROM by JG1CCL W3CCL
  
//lcd.init();                       //-----1R0 I2C LCD by JG1CCL W3CCL
//lcd.backlight();                  //-----1R0 I2C LCD by JG1CCL W3CCL

//-----1R0 BasicGoertzelCal() by JG1CCLW3CCL
//  ////////////////////////////////////
//  // The basic goertzel calculation //
//  ////////////////////////////////////
//  int k;
//  float omega;
//  k = (int)(0.5+((n*target_freq)/sampling_freq));
//  omega = (2.0*PI*k)/n;
//  sine = sin(omega);
//  cosine = cos(omega);
//  coeff = 2.0*cosine;
//-----1R0 BasicGoertzelCal() by JG1CCL W3CCL

  ///////////////////////////////
  // define special characters //
  ///////////////////////////////
  lcd.createChar(0, U_umlaut);    // German
  lcd.createChar(1, O_umlaut);    // German, Swedish
  lcd.createChar(2, A_umlaut);    // German, Swedish
  lcd.createChar(3, AE_capital);  // Danish, Norwegian
  lcd.createChar(4, OE_capital);  // Danish, Norwegian
  lcd.createChar(5, fullblock);
  lcd.createChar(6, AA_capital);  // Danish, Norwegian, Swedish
  lcd.createChar(7, emtyblock);
  lcd.clear();

  Serial.begin(115200); 
//-----1R0 Title by JG1CCL W3CCL  
//  pinMode(ledPin, OUTPUT);
  lcd.begin(colums, rows);
  lcd.print("CW Decoder");
  lcd.setCursor(0, 1);
  lcd.print("HHE-114 1.01-1R5");
  Serial.println("HHE-114 CW Decoder 1.01-1R5");  //-----1R5 by JG1CCL W3CCL
  delay(3000);
  BasicGoertzelCal(); // The basic goertzel calculation by JG1CCL W3CCL
//-----1R0 Titke by JG1CCL W3CCL

  for(int index = 0; index < colums; index++){
    line1[index] = 32;
    line2[index] = 32;
  }
}

///////////////
// main loop //
///////////////
void loop() {
//-----1R0 SW by JG1CCL W3CCL
  float lacktime = 1;
//  if(digitalRead(2) == HIGH){ // スイッチの状態を調べる
//    digitalWrite(13,HIGH);    // スイッチが押されているならLEDを点灯
//    sw = 1; // 和文に強制的に切り替える
//  }
//  if(digitalRead(3) == HIGH){ // スイッチの状態を調べる
//    digitalWrite(13,HIGH);    // スイッチが押されているならLEDを点灯
//    sw = 0; // 欧文に強制的に切り替える
//  }
//  else{                   // この行を含む、以下3行をコメントアウトしないと和文切替時にLEDが点かない
//   digitalWrite(13,LOW);  // スイッチが押されていないならLEDを消灯
//  }

//-----1R3 Chattering prevention and status display by JG1CCL W3CCL
  // SW1ボタンを押したときには、
  if(digitalRead(sw1Pin) == LOW){
    digitalWrite(ledPin, HIGH); // LEDを点灯する
    capacity++;                 // Capacity counter increment
//    Serial.println(capacity);
    if(capacity > PUSH_SHORT){  // Short press push button
      buttonFlag1 = FLAG_ON;    // ボタンを押したフラグをONにする   
    }
  }
  // それ以外で、SW1ボタンを離したときには、
  else if(buttonFlag1 == FLAG_ON){
    digitalWrite(ledPin, LOW);  // LEDを消灯する
    if(capacity > PUSH_LONG){   // Long press push button 
      sw = !sw;                 // 強制的に切り替える（欧文⇔和文）
      EEPROM.write(EEPROM_ADDRESS_SW1, sw); //-----1R4 EEPROM Address SW1 by JG1CCL W3CCL
    }
    BasicGoertzelCal();         // The basic goertzel calculation
    buttonFlag1 = FLAG_OFF;     // ボタンを押したフラグをOFFにする
    capacity = 0;               // Capacity counter initialization 
  }

  // SW2ボタンを押したときには、
  if(digitalRead(sw2Pin) == LOW){
    digitalWrite(ledPin, HIGH); // LEDを点灯する
    capacity++;                 // Capacity counter increment
    if(capacity > PUSH_SHORT){  // Short press push button
      buttonFlag2 = FLAG_ON;    // ボタンを押したフラグをONにする
    }
  }
  // それ以外で、SW2ボタンを離したときには、
  else if(buttonFlag2 == FLAG_ON){
    digitalWrite(ledPin, LOW);  // LEDを消灯する
    if(capacity > PUSH_LONG){   // Long press push button
      if(++sw2 >= 4)sw2 = 0;    // 配列要素数を超えたときには0にする
      EEPROM.write(EEPROM_ADDRESS_SW2, sw2);  //-----1R4 EEPROM Address SW2 by JG1CCL W3CCL
      target_freq = target_freqArray[sw2];
    }
    BasicGoertzelCal();         // The basic goertzel calculation
    buttonFlag2 = FLAG_OFF;     // ボタンを押したフラグをOFFにする
    capacity = 0;               // Capacity counter initialization
  }

  // SW3ボタンを押したときには、
  if(digitalRead(sw3Pin) == LOW){
    digitalWrite(ledPin, HIGH); // LEDを点灯する
    capacity++;                 // Capacity counter increment
    if(capacity > PUSH_SHORT){  // Short press push button
      buttonFlag3 = FLAG_ON;    // ボタンを押したフラグをONにする
    }
  }
  // それ以外で、SW3ボタンを離したときには、
  else if(buttonFlag3 == FLAG_ON){
    digitalWrite(ledPin, LOW);  // LEDを消灯する
    if(capacity > PUSH_LONG){   // Long press push button
      if(++sw3 >= 4)sw3 = 0;    // 配列要素数を超えたときには0にする
      EEPROM.write(EEPROM_ADDRESS_SW3, sw3);  //-----1R4 EEPROM Address SW3 by JG1CCL W3CCL
      n = nArray[sw3];
    }
    BasicGoertzelCal();         // The basic goertzel calculation
    buttonFlag3 = FLAG_OFF;     // ボタンを押したフラグをOFFにする
    capacity = 0;               // Capacity counter initialization
  }
//-----1R3 Chattering prevention and status display by JG1CCL W3CCL
//-----1R0 SW by JG1CCL W3CCL

  ///////////////////////////////////// 
  // The basic where we get the tone //
  /////////////////////////////////////
  for(char index = 0; index < n; index++){
    testData[index] = analogRead(audioInPin);
  }

  for(char index = 0; index < n; index++){
    float Q0;
    Q0 = coeff*Q1-Q2+(float)testData[index];
    Q2 = Q1;
    Q1 = Q0;  
  }
  float magnitudeSquared = (Q1*Q1)+(Q2*Q2)-Q1*Q2*coeff; // we do only need the real part
  magnitude = sqrt(magnitudeSquared);
  Q2 = 0;
  Q1 = 0;

  //Serial.print(magnitude); Serial.println();  //  here you can measure magnitude for setup..
  
  /////////////////////////////////////////////////////////// 
  // here we will try to set the magnitude limit automatic //
  ///////////////////////////////////////////////////////////
  if(magnitude > magnitudelimit_low){
    magnitudelimit = (magnitudelimit+((magnitude-magnitudelimit)/6)); // moving average filter
  }

  if(magnitudelimit < magnitudelimit_low)
    magnitudelimit = magnitudelimit_low;

  ////////////////////////////////////
  // now we check for the magnitude //
  ////////////////////////////////////
  if(magnitude > magnitudelimit*0.6)  // just to have some space up
    realstate = HIGH;
  else
    realstate = LOW;

  ///////////////////////////////////////////////////// 
  // here we clean up the state with a noise blanker //
  /////////////////////////////////////////////////////
  if(realstate != realstatebefore){
    laststarttime = millis();
  }
  if((millis()-laststarttime) > nbtime){
    if(realstate != filteredstate){
    filteredstate = realstate;
    }
  }

  ////////////////////////////////////////////////////////////
  // Then we do want to have some durations on high and low //
  ////////////////////////////////////////////////////////////
  if(filteredstate != filteredstatebefore){
    if(filteredstate == HIGH){
      starttimehigh = millis();
      lowduration = (millis() - startttimelow);
    }
    if(filteredstate == LOW){
      startttimelow = millis();
      highduration = (millis()-starttimehigh);
      if(highduration < (2*hightimesavg) || hightimesavg == 0){
        hightimesavg = (highduration+hightimesavg+hightimesavg)/3;
          // now we know avg dit time ( rolling 3 avg)
      }
      if(highduration > (5*hightimesavg)){
        hightimesavg = highduration+hightimesavg; // if speed decrease fast ..
      }
    }
  }

  ///////////////////////////////////////////////////////////////
  // now we will check which kind of baud we have - dit or dah //
  // and what kind of pause we do have 1 - 3 or 7 pause        //
  // we think that hightimeavg = 1 bit                         //
  ///////////////////////////////////////////////////////////////
  if(filteredstate != filteredstatebefore){
    stop = LOW;
    if(filteredstate == LOW){  // we did end a HIGH
      if(highduration < (hightimesavg*2) && highduration > (hightimesavg*0.6)){
        // 0.6 filter out false dits
        strcat(code,".");
//        Serial.print(".");  //-----1R5 by JG1CCL W3CCL
      }
      if(highduration > (hightimesavg*2) && highduration < (hightimesavg*6)){ 
        strcat(code,"-");
//        Serial.print("-");  //-----1R5 by JG1CCL W3CCL
        wpm = (wpm + (1200/((highduration)/3)))/2;  // the most precise we can do ;o)
      }
    }

    if(filteredstate == HIGH){ // we did end a LOW
//      float lacktime = 1; //-----1R2 by JG1CCL W3CCL
      if(wpm > 25)lacktime = 1.0;
        // when high speeds we have to have a little more pause before new letter or new word
      if(wpm > 30)lacktime = 1.2;
      if(wpm > 35)lacktime = 1.5;
      if(lowduration > (hightimesavg*(2*lacktime)) && lowduration < hightimesavg*(5*lacktime)){ 
        // letter space
//        docode();       //-----1R2 by JG1CCL W3CCL
//        code[0] = '\0'; //-----1R2 by JG1CCL W3CCL
//        Serial.print("/");  //-----1R5 by JG1CCL W3CCL
      }
      if(lowduration >= hightimesavg*(5*lacktime)){  // word space
//        docode();       //-----1R2 by JG1CCL W3CCL
//        code[0] = '\0'; //-----1R2 by JG1CCL W3CCL
        printascii(32);
//        Serial.println();  //-----1R5 by JG1CCL W3CCL
      }
    }
  }

  //////////////////////////////////////////////////////////////
  //-----1R2 letter space, word space                         //
  // For letter or word delimiters, display the letter first  //
  //                                          by JG1CCL W3CCL //
  //////////////////////////////////////////////////////////////
  if(filteredstate == LOW && (millis()-startttimelow) > (hightimesavg*(2*lacktime))){
    docode();
    code[0] = '\0';
  }

  //////////////////////////////
  // write if no more letters //
  //////////////////////////////
  if((millis()-startttimelow) > (highduration*6) && stop == LOW){
    docode();
    code[0] = '\0';
    stop = HIGH;
  }

  /////////////////////////////////////
  // we will turn on and off the LED //
  // and the speaker                 //
  /////////////////////////////////////
  if(filteredstate == HIGH){
    digitalWrite(ledPin, HIGH);
    tone(audioOutPin,target_freq);
//    tone(audioOutPin,530);
  }
  else{
    digitalWrite(ledPin, LOW);
    noTone(audioOutPin);
  }

  //////////////////////////////////
  // the end of main loop clean up//        
  /////////////////////////////////
  updateinfolinelcd();
  realstatebefore = realstate;
  lasthighduration = highduration;
  filteredstatebefore = filteredstate;
}

////////////////////////////////
// translate cw code to ascii //
////////////////////////////////
void docode(){
  if(strcmp(code,"-..---") == 0) sw = 1;  // 和文
  if(strcmp(code,"...-.") == 0) sw = 0;   // 欧文

  if(sw == 0){
    if(strcmp(code,".-..-.") == 0){printascii(0x29); sw = 1;}  // 閉じカッコの後は和文に戻る

    if(strcmp(code,".-") == 0) printascii(65);
    if(strcmp(code,"-...") == 0) printascii(66);
    if(strcmp(code,"-.-.") == 0) printascii(67);
    if(strcmp(code,"-..") == 0) printascii(68);
    if(strcmp(code,".") == 0) printascii(69);
    if(strcmp(code,"..-.") == 0) printascii(70);
    if(strcmp(code,"--.") == 0) printascii(71);
    if(strcmp(code,"....") == 0) printascii(72);
    if(strcmp(code,"..") == 0) printascii(73);
    if(strcmp(code,".---") == 0) printascii(74);
    if(strcmp(code,"-.-") == 0) printascii(75);
    if(strcmp(code,".-..") == 0) printascii(76);
    if(strcmp(code,"--") == 0) printascii(77);
    if(strcmp(code,"-.") == 0) printascii(78);
    if(strcmp(code,"---") == 0) printascii(79);
    if(strcmp(code,".--.") == 0) printascii(80);
    if(strcmp(code,"--.-") == 0) printascii(81);
    if(strcmp(code,".-.") == 0) printascii(82);
    if(strcmp(code,"...") == 0) printascii(83);
    if(strcmp(code,"-") == 0) printascii(84);
    if(strcmp(code,"..-") == 0) printascii(85);
    if(strcmp(code,"...-") == 0) printascii(86);
    if(strcmp(code,".--") == 0) printascii(87);
    if(strcmp(code,"-..-") == 0) printascii(88);
    if(strcmp(code,"-.--") == 0) printascii(89);
    if(strcmp(code,"--..") == 0) printascii(90);

    if(strcmp(code,".----") == 0) printascii(49);
    if(strcmp(code,"..---") == 0) printascii(50);
    if(strcmp(code,"...--") == 0) printascii(51);
    if(strcmp(code,"....-") == 0) printascii(52);
    if(strcmp(code,".....") == 0) printascii(53);
    if(strcmp(code,"-....") == 0) printascii(54);
    if(strcmp(code,"--...") == 0) printascii(55);
    if(strcmp(code,"---..") == 0) printascii(56);
    if(strcmp(code,"----.") == 0) printascii(57);
    if(strcmp(code,"-----") == 0) printascii(48);

    if(strcmp(code,"..--..") == 0) printascii(63);
    if(strcmp(code,".-.-.-") == 0) printascii(46);
    if(strcmp(code,"--..--") == 0) printascii(44);
    if(strcmp(code,"-.-.--") == 0) printascii(33);
    if(strcmp(code,".--.-.") == 0) printascii(64);
    if(strcmp(code,"---...") == 0) printascii(58);
    if(strcmp(code,"-....-") == 0) printascii(45);
    if(strcmp(code,"-..-.") == 0) printascii(47);

    if(strcmp(code,"-.--.") == 0) printascii(40);
    if(strcmp(code,"-.--.-") == 0) printascii(41);
    if(strcmp(code,".-...") == 0) printascii(95);
    if(strcmp(code,"...-..-") == 0) printascii(36);
    if(strcmp(code,"...-.-") == 0) printascii(42);
    if(strcmp(code,".-.-.") == 0) printascii(60);
//-----1R1 欧文←和文 by JG1CCL W3CCL
//    if(strcmp(code,"...-.") == 0) printascii(126);
    if(strcmp(code,"...-.") == 0) printascii(127);  // 欧文←和文
//-----1R1 欧文←和文 by JG1CCL W3CCL

  //////////////////
  // The specials //
  //////////////////
  if(strcmp(code,".-.-") == 0) printascii(3);
  if(strcmp(code,"---.") == 0) printascii(4);
  if(strcmp(code,".--.-") == 0) printascii(6);
  }

  if(sw == 1){ // wabun
    if(strcmp(code,"-.--.-") == 0) {printascii(0x28); sw = 0;} // カッコの後は欧文

    if(strcmp(code,"--.--") == 0) printascii(0xB1);
    if(strcmp(code,".-") == 0) printascii(0xB2);
    if(strcmp(code,"..-") == 0) printascii(0xB3);
    if(strcmp(code,"-.---") == 0) printascii(0xB4);
    if(strcmp(code,".-...") == 0) printascii(0xB5);
    if(strcmp(code,".-..") == 0) printascii(0xB6);
    if(strcmp(code,"-.-..") == 0) printascii(0xB7);
    if(strcmp(code,"...-") == 0) printascii(0xB8);
    if(strcmp(code,"-.--") == 0) printascii(0xB9);
    if(strcmp(code,"----") == 0) printascii(0xBA);
    if(strcmp(code,"-.-.-") == 0) printascii(0xBB);
    if(strcmp(code,"--.-.") == 0) printascii(0xBC);
    if(strcmp(code,"---.-") == 0) printascii(0xBD);
    if(strcmp(code,".---.") == 0) printascii(0xBE);
    if(strcmp(code,"---.") == 0) printascii(0xBF);
    if(strcmp(code,"-.") == 0) printascii(0xC0);
    if(strcmp(code,"..-.") == 0) printascii(0xC1);
    if(strcmp(code,".--.") == 0) printascii(0xC2);
    if(strcmp(code,".-.--") == 0) printascii(0xC3);
    if(strcmp(code,"..-..") == 0) printascii(0xC4);
    if(strcmp(code,".-.") == 0) printascii(0xC5);
    if(strcmp(code,"-.-.") == 0) printascii(0xC6);
    if(strcmp(code,"....") == 0) printascii(0xC7);
    if(strcmp(code,"--.-") == 0) printascii(0xC8);
    if(strcmp(code,"..--") == 0) printascii(0xC9);
    if(strcmp(code,"-...") == 0) printascii(0xCA);
    if(strcmp(code,"--..-") == 0) printascii(0xCB);
    if(strcmp(code,"--..") == 0) printascii(0xCC);
    if(strcmp(code,".") == 0) printascii(0xCD);
    if(strcmp(code,"-..") == 0) printascii(0xCE);
    if(strcmp(code,"-..-") == 0) printascii(0xCF);
    if(strcmp(code,"..-.-") == 0) printascii(0xD0);
    if(strcmp(code,"-") == 0) printascii(0xD1);
    if(strcmp(code,"-...-") == 0) printascii(0xD2);
    if(strcmp(code,"-..-.") == 0) printascii(0xD3);
    if(strcmp(code,".--") == 0) printascii(0xD4);
    if(strcmp(code,"-..--") == 0) printascii(0xD5);
    if(strcmp(code,"--") == 0) printascii(0xD6);
    if(strcmp(code,"...") == 0) printascii(0xD7);
    if(strcmp(code,"--.") == 0) printascii(0xD8);
    if(strcmp(code,"-.--.") == 0) printascii(0xD9);
    if(strcmp(code,"---") == 0) printascii(0xDA);
    if(strcmp(code,".-.-") == 0) printascii(0xDB);
    if(strcmp(code,"-.-") == 0) printascii(0xDC);
    if(strcmp(code,".-.-.") == 0) printascii(0xDD);
    if(strcmp(code,"..") == 0) printascii(0xDE);
    if(strcmp(code,"..--.") == 0) printascii(0xDF);
    if(strcmp(code,".---") == 0) printascii(0xA6);

    if(strcmp(code,".----") == 0) printascii(49);
    if(strcmp(code,"..---") == 0) printascii(50);
    if(strcmp(code,"...--") == 0) printascii(51);
    if(strcmp(code,"....-") == 0) printascii(52);
    if(strcmp(code,".....") == 0) printascii(53);
    if(strcmp(code,"-....") == 0) printascii(54);
    if(strcmp(code,"--...") == 0) printascii(55);
    if(strcmp(code,"---..") == 0) printascii(56);
    if(strcmp(code,"----.") == 0) printascii(57);
    if(strcmp(code,"-----") == 0) printascii(48);

    if(strcmp(code,"..--..") == 0) printascii(63);
    //if(strcmp(code,".-.-.-") == 0) printascii(46);
    if(strcmp(code,"--..--") == 0) printascii(44);
    if(strcmp(code,"-.-.--") == 0) printascii(33);
    if(strcmp(code,".--.-.") == 0) printascii(64);
    if(strcmp(code,"---...") == 0) printascii(58);
    if(strcmp(code,"-....-") == 0) printascii(45);
    //if(strcmp(code,"-..-.") == 0) printascii(47);

    //if(strcmp(code,"-.--.") == 0) printascii(40);
    //if(strcmp(code,"-.--.-") == 0) printascii(41);
    //if(strcmp(code,".-...") == 0) printascii(95);
    //if(strcmp(code,"...-..-") == 0) printascii(36);
    if(strcmp(code,"...-.-") == 0) printascii(42);
    //if(strcmp(code,".-.-.") == 0) printascii(60);
    //if(strcmp(code,"...-.") == 0) printascii(126);
    if(strcmp(code,".--.-") == 0) printascii(45);
    if(strcmp(code,"-..---") == 0) printascii(126); //-----1R1 欧文→和文 by JG1CCL W3CCL
    
    //////////////////
    // The specials //
    //////////////////
    if(strcmp(code,".-.-..") == 0) printascii(0xA3);
    if(strcmp(code,".-.-.-") == 0) printascii(0xA4);
    //if(strcmp(code,".-.-") == 0) printascii(3);
    //if(strcmp(code,"---.") == 0) printascii(4);
    //if(strcmp(code,".--.-") == 0) printascii(6);
  }
}

/////////////////////////////////////
// print the ascii code to the lcd //
// one a time so we can generate   //
// special letters                 //
/////////////////////////////////////
void printascii(int asciinumber){
  int fail = 0;

  Serial.write(asciinumber);  //-----1R5 by JG1CCL W3CCL

  if(rows == 4 and colums == 16)fail = -4;
    // to fix the library problem with 4*16 display http://forum.arduino.cc/index.php/topic,14604.0.html
  if(lcdindex > colums-1){
    lcdindex = 0;
    if(rows == 4){
      for(int i = 0; i <= colums-1 ; i++){
        lcd.setCursor(i,rows-3);
        lcd.write(line2[i]);
        line2[i] = line1[i];
      }
    }
    for(int i = 0; i <= colums-1 ; i++){
      lcd.setCursor(i+fail, rows-2);
      lcd.write(line1[i]);
      lcd.setCursor(i+fail, rows-1);
      lcd.write(32);
    }
  }
  line1[lcdindex]=asciinumber;
  lcd.setCursor(lcdindex+fail,rows-1);
  lcd.write(asciinumber);
  lcdindex += 1;
}

/////////////////////////////////////
// here we update the upper line   //
// with the speed.                 //
/////////////////////////////////////
void updateinfolinelcd(){
//-----1R0 LCD & BasicGoertzelCal() by JG1CCL W3CCL
//  int place;

//  if(rows == 4){
//    place = colums/2;
//  }
//  else{
//    place = 2;
//  }

//  if(wpm < 10){
//    lcd.setCursor((place)-2,0);
//    lcd.print("0");
//    lcd.setCursor((place)-1,0);
//    lcd.print(wpm);
//    lcd.setCursor((place),0);
//    if(sw == 0){
//      lcd.print(" WPM [A] ");
//    }
//    else{
//      lcd.print(" WPM [W] ");
//    }
//  }
//  else{
//    lcd.setCursor((place)-2,0);
//    lcd.print(wpm);
//    lcd.setCursor((place),0);
//    if(sw == 0){
//      lcd.print(" WPM [A] ");
//    }
//    else{
//      lcd.print(" WPM [W] ");
//    }
//  }

  lcd.setCursor(0, 0);
  if(wpm < 10)lcd.print("0");
  lcd.print(wpm);
  if(colums == 20)lcd.print("WPM"); // 20桁のときにはWPMを表示する
  lcd.print(" ");
}

////////////////////////////////////
// The basic goertzel calculation //
////////////////////////////////////
void BasicGoertzelCal(){
  int k;
  float omega;

  k = (int)(0.5+((n*target_freq)/sampling_freq));
  omega = (2.0*PI*k)/n;
  sine = sin(omega);
  cosine = cos(omega);
  coeff = 2.0*cosine;
  bw = (sampling_freq/n)+0.5; // Bandwidth

  lcd.setCursor(6, 0);
  if(colums != 20)lcd.setCursor(3, 0);
  if(sw == 0){
    lcd.print("A ");
  }
  else{
    lcd.print("W ");
  }
  lcd.print((int)target_freq);
  lcd.print("TF ");
  if(bw < 100)lcd.print("0");
  lcd.print((int)bw);
  lcd.print("BW");
}
//-----1R0 LCD & BasicGoertzelCal() by JG1CCL W3CCL
// End
